//     This file is part of cdloop
//     Copyright (C) 2001-2003 Claus Brunzema <mail@cbrunzema.de>
// 
//     This program is free software; you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation; either version 2 of the License, or
//     (at your option) any later version.
// 
//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with this program; if not, write to the Free Software
//     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// 
#include "CDDisc.hh"

#include <cstdio>
#include <iostream>

CDDisc::
CDDisc() :
        was_changed(true)
{ }

void 
CDDisc::
clear()
{
    if(tracks.empty())
	return;
    tracks.clear();
    was_changed=true;
}

void
CDDisc::
push_back(const CDAbsPos& track_start)
{
    tracks.push_back(track_start);
    was_changed=true;
}

unsigned int
CDDisc::
no_of_tracks() const
{
    if(tracks.empty()) return 0;
    return tracks.size()-1; // leadout is additional 'track'
}

CDAbsPos
CDDisc::
abs_pos(const CDRelPos& rel_pos) const
{
    CDAbsPos ret;
    ret.clear();
    if(tracks.empty()) return ret;

    ret=tracks[rel_pos.track-1];
    ret.minute+=rel_pos.minute;
    ret.second+=rel_pos.second;
    ret.frame+=rel_pos.frame;
    ret.normalize();
    if(ret<tracks[0] || ret>end())
        ret.clear();
    return ret;
}

CDRelPos
CDDisc::
rel_pos(const CDAbsPos& abs_pos) const
{
    CDRelPos ret;
    ret.clear();
    unsigned int track=0;
    while(track<tracks.size() && abs_pos>=tracks[track])
        track++;
    if(track==0 || track>=tracks.size())
        return ret;
    ret.track=track;
    ret.minute=(abs_pos-tracks[track-1]).minute;
    ret.second=(abs_pos-tracks[track-1]).second;
    ret.frame=(abs_pos-tracks[track-1]).frame;
    return ret;
}

CDAbsPos
CDDisc::
end() const
{
    CDAbsPos ret;
    ret.clear();
    if(!tracks.empty())
        ret=tracks.back();
    return ret;
}

unsigned int
CDDisc::
id() const
{
    if(tracks.empty()) return 0;
    
	// this algorithm comes from xmcd-2.6
	// Copyright (C) 1993-2000  Ti Kan
	// available at http://metalab.unc.edu/tkan/xmcd/
    
    int t=0;
    int n=0;

    for(unsigned int i=0; i<no_of_tracks(); i++) {
        char buf[16];
        char* p;
        snprintf(buf, 15, "%u", (tracks[i].minute*60)+tracks[i].second);
        for(p=buf; *p!='\0'; p++) 
            n+=(*p-'0');

        t+=((tracks[i+1].minute*60)+tracks[i+1].second)-
            ((tracks[i].minute*60)+tracks[i].second);
    }
    return ((n % 0xff) << 24 | t << 8 | no_of_tracks());
}

bool
CDDisc::
valid(const CDRelPos& pos) const
{
    if(!pos.valid()) return false;
    return abs_pos(pos).valid();
}    

bool
CDDisc::
changed()
{
    bool ret=was_changed;
    was_changed=false;
    return ret;
}

void
CDDisc::
debug_print() const
{
    for(unsigned int i=0; i<tracks.size(); i++)
        tracks[i].debug_print();
}
