//     This file is part of cdloop
//     Copyright (C) 2001-2003 Claus Brunzema <mail@cbrunzema.de>
// 
//     This program is free software; you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation; either version 2 of the License, or
//     (at your option) any later version.
// 
//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with this program; if not, write to the Free Software
//     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// 
#ifndef CDPLAYER_HH_INCLUDED
#define CDPLAYER_HH_INCLUDED

#include <unistd.h>
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/time.h>

#include <string>
#include "Loop.hh"
#include "built_Status.hh"
#include "CDAbsPos.hh"
#include "CDRelPos.hh"
#include "CDDrive.hh"

class CDPlayer
{
private:
    enum Cmd {
        CMD_NO_CMD,
        CMD_INIT,
        CMD_QUIT,
        CMD_PLAY,
        CMD_SET_LOOP,
        CMD_CLEAR_LOOP,
        CMD_GET_ELAPSED_GAP,
        CMD_GET_STATUS,
        CMD_GET_NO_OF_TRACKS,
        CMD_GET_ID,
        CMD_GET_POS,
        CMD_PAUSE,
        CMD_RESUME,
        CMD_STOP,
        CMD_EJECT,
        CMD_CLOSE_TRAY,
        CMD_ADD_POS,
        CMD_GET_TOC_CHANGED
    };
    struct Shmblock            // structure of the shared memory area
    {
        volatile Cmd cmd;
        union {
            CDRelPos pos;      // for PLAY
            Loop loop;         // for PLAY_LOOP
            long offset;       // for SEEK
            struct {
                CDRelPos base;
                CDAbsPos offset;
            } add_pos;         // for ADD_POS
        } cmd_data;
        union {
            Status status;
            long elapsed_gap;
            unsigned int no_of_tracks;
            unsigned int id;
            CDRelPos pos;
            bool toc_changed;
        } resp_data;
    };

    Status error;
    pid_t control_child;  // process id for the controller
    
    int shmid;            // id of the shared memory area
    Shmblock* shm;
    CDDrive drive;

    Loop loop;
    struct timeval gap_start;
    
    Status player_status();
    void controller();
    void send_command(Cmd cmd);
    void wait_for_controller();

    void clear_gap_start();
    void clear_loop_vars();
    void finish_gap();
    
public:
    CDPlayer(const std::string device);
    ~CDPlayer();

    void play(const CDRelPos& pos);
    void set_loop(const Loop& loop);
    void clear_loop();
    long elapsed_gap();
    Status status();
    unsigned int no_of_tracks();
    unsigned int id();
    CDRelPos pos();
    void pause();
    void resume();
    void stop();
    void eject();
    void close_tray();
    CDRelPos add_pos(const CDRelPos& base, const CDAbsPos& offset);
    bool toc_changed();
};

#endif
