//     This file is part of cdloop
//     Copyright (C) 2001-2003 Claus Brunzema <mail@cbrunzema.de>
// 
//     This program is free software; you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation; either version 2 of the License, or
//     (at your option) any later version.
// 
//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with this program; if not, write to the Free Software
//     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// 
#ifndef CDRELPOS_HH_INCLUDED
#define CDRELPOS_HH_INCLUDED

#include <iostream>

struct CDRelPos
{
    int track;
    int minute;
    int second;
    int frame;

    void set(const int track_, const int minute_,
             const int second_, const int frame_)
        {
            track=track_;
            minute=minute_;
            second=second_;
            frame=frame_;
        }
    void clear()
        {
            set(-1, -1, -1, -1);
        }
    bool valid() const
        {
            return track>0 && minute>=0 && second>=0 && frame>=0;
        }
    bool operator==(const CDRelPos& rv) const
        {
            return track==rv.track &&
                minute==rv.minute &&
                second==rv.second &&
                frame==rv.frame;
        }
    bool operator<(const CDRelPos& rv) const
        {
            return
                (track<rv.track) ||
                (track==rv.track && minute<rv.minute) ||
                (track==rv.track && minute==rv.minute && second<rv.second) ||
                (track==rv.track && minute==rv.minute &&
                 second==rv.second && frame<rv.frame);
        }
    bool operator<=(const CDRelPos& rv) const
        {
            return *this<rv || *this==rv;
        }
    bool operator>(const CDRelPos& rv) const
        {
            return !(*this <= rv);
        }
    bool operator>=(const CDRelPos& rv) const
        {
            return !(*this < rv);
        }

    void debug_print() const
        {
            std::cout << "CDRelPos:"
                      << track << "-"
                      << minute << ":" << second << ":" << frame
                      << std::endl;
        }
};

#endif
