//     This file is part of cdloop
//     Copyright (C) 2001-2003 Claus Brunzema <mail@cbrunzema.de>
// 
//     This program is free software; you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation; either version 2 of the License, or
//     (at your option) any later version.
// 
//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with this program; if not, write to the Free Software
//     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// 
#ifdef AMIXER
#include "Mixer.hh"

#include <cstdlib>
#include <cstdio>


//-------------------------------------------------------------------------
void set_vol(int card_number, double left, double right)
{
    static char buffer[128];
        
    snprintf(buffer,
             127,
             "amixer -q -c %i sset 'Master',0 front %u%%,%u%%",
             card_number,
             static_cast<unsigned int>(100 * left),
             static_cast<unsigned int>(100 * right));
    system(buffer);
}


Mixer::
Mixer() : _card_number(0),
          _volume_left(80.0),
          _mute_left(false),
          _volume_right(80.0),
          _mute_right(false)
{ }

Mixer::
Mixer(const int card_number) :
        _card_number(card_number),
        _volume_left(80.0),
        _mute_left(false),
        _volume_right(80.0),
        _mute_right(false)
{ }

void
Mixer::
set_master_volume_front_left(double level)
{
    _volume_left = level;
    if(!_mute_left) {
        set_vol(_card_number, _volume_left, _mute_right ? 0.0 : _volume_right);
    }
}

void 
Mixer::
set_mute_front_left(bool mute)
{
    _mute_left = mute;
    set_vol(_card_number,
            _mute_left ? 0.0 : _volume_left,
            _mute_right ? 0.0 : _volume_right);
}

void
Mixer::
set_master_volume_front_right(double level)
{
    _volume_right = level;
    if(!_mute_right) {
        set_vol(_card_number, _mute_left ? 0.0 :_volume_left, _volume_right);
    }
}

void 
Mixer::
set_mute_front_right(bool mute)
{
    _mute_right = mute;
    set_vol(_card_number,
            _mute_left ? 0.0 : _volume_left,
            _mute_right ? 0.0 : _volume_right);
}

#endif
