//     This file is part of cdloop
//     Copyright (C) 2001-2003 Claus Brunzema <mail@cbrunzema.de>
// 
//     This program is free software; you can redistribute it and/or modify
//     it under the terms of the GNU General Public License as published by
//     the Free Software Foundation; either version 2 of the License, or
//     (at your option) any later version.
// 
//     This program is distributed in the hope that it will be useful,
//     but WITHOUT ANY WARRANTY; without even the implied warranty of
//     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//     GNU General Public License for more details.
// 
//     You should have received a copy of the GNU General Public License
//     along with this program; if not, write to the Free Software
//     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
// 
#include <cstdlib>
#include <guile/gh.h>

#include "CDPlayer.hh"
#include "CDAbsPos.hh"
#include "CDRelPos.hh"
#include "Loop.hh"
#include "built_Status_to_symbol.hh"

#include "Mixer.hh"

#include "built_cdloop_script.h"

CDPlayer* cd=0;
Mixer* mixer=0;

void cleanup()
{
    if(cd) delete cd;
    cd=0;
    if(mixer) delete mixer;
    mixer=0;
}

SCM cdplayer_init(SCM device)
{
    char* device_=gh_scm2newstr(device, 0);
    gh_defer_ints();
    cd=new CDPlayer(device_);
    gh_allow_ints();
    scm_must_free(device_);
    return SCM_UNSPECIFIED;
}    

SCM cdplayer_exit()
{
    gh_defer_ints();
    if(cd) delete cd;
    cd=0;
    gh_allow_ints();
    return SCM_UNSPECIFIED;
}

SCM cdplayer_play(SCM pos)
{
    CDRelPos pos_;
    pos_.set(gh_scm2int(gh_list_ref(pos, gh_int2scm(0))),
             gh_scm2int(gh_list_ref(pos, gh_int2scm(1))),
             gh_scm2int(gh_list_ref(pos, gh_int2scm(2))),
             gh_scm2int(gh_list_ref(pos, gh_int2scm(3))));
    gh_defer_ints();
    cd->play(pos_);
    gh_allow_ints();
    return SCM_UNSPECIFIED;
}

SCM cdplayer_set_loop(SCM loop)
{
    Loop loop_;
    SCM start=gh_car(loop);
    SCM end=gh_cadr(loop);
    SCM gap=gh_caddr(loop);

    loop_.start.set(gh_scm2int(gh_list_ref(start, gh_int2scm(0))),
                    gh_scm2int(gh_list_ref(start, gh_int2scm(1))),
                    gh_scm2int(gh_list_ref(start, gh_int2scm(2))),
                    gh_scm2int(gh_list_ref(start, gh_int2scm(3))));
    loop_.end.set(gh_scm2int(gh_list_ref(end, gh_int2scm(0))),
                    gh_scm2int(gh_list_ref(end, gh_int2scm(1))),
                    gh_scm2int(gh_list_ref(end, gh_int2scm(2))),
                    gh_scm2int(gh_list_ref(end, gh_int2scm(3))));
    loop_.gap=gh_scm2long(gap);
    gh_defer_ints();
    cd->set_loop(loop_);
    gh_allow_ints();
    return SCM_UNSPECIFIED;
}

SCM cdplayer_clear_loop()
{
    gh_defer_ints();
    cd->clear_loop();
    gh_allow_ints();
    return SCM_UNSPECIFIED;
}

SCM cdplayer_elapsed_gap()
{
    gh_defer_ints();
    long ret=cd->elapsed_gap();
    gh_allow_ints();
    return gh_long2scm(ret);
}

SCM cdplayer_status()
{
    gh_defer_ints();
    Status status=cd->status();
    gh_allow_ints();
    return Status_to_symbol(status);
}

SCM cdplayer_no_of_tracks()
{
    gh_defer_ints();
    unsigned int n=cd->no_of_tracks();
    gh_allow_ints();
    return gh_int2scm(n);
}

SCM cdplayer_id()
{
    gh_defer_ints();
    unsigned int id=cd->id();
    gh_allow_ints();
    if(id==0)
        return SCM_BOOL_F;
    else
        return gh_long2scm(static_cast<long>(id));
}

SCM cdplayer_pos()
{
    gh_defer_ints();
    CDRelPos pos=cd->pos();
    gh_allow_ints();

    if(!pos.valid()) return SCM_BOOL_F;
    int pos_array[4];
    pos_array[0]=pos.track;
    pos_array[1]=pos.minute;
    pos_array[2]=pos.second;
    pos_array[3]=pos.frame;
    return gh_vector_to_list(gh_ints2scm(pos_array, 4));
}

SCM cdplayer_pause()
{
    gh_defer_ints();
    cd->pause();
    gh_allow_ints();
    return SCM_UNSPECIFIED;
}

SCM cdplayer_resume()
{
    gh_defer_ints();
    cd->resume();
    gh_allow_ints();
    return SCM_UNSPECIFIED;
}

SCM cdplayer_stop()
{
    gh_defer_ints();
    cd->stop();
    gh_allow_ints();
    return SCM_UNSPECIFIED;
}

SCM cdplayer_eject()
{
    gh_defer_ints();
    cd->eject();
    gh_allow_ints();
    return SCM_UNSPECIFIED;
}

SCM cdplayer_close_tray()
{
    gh_defer_ints();
    cd->close_tray();
    gh_allow_ints();
    return SCM_UNSPECIFIED;
}

SCM cdplayer_add_pos(SCM base, SCM offset)
{
    if(!gh_scm2bool(base) || !gh_scm2bool(offset))
        return SCM_BOOL_F;

    CDRelPos base_;
    base_.set(gh_scm2int(gh_list_ref(base, gh_int2scm(0))),
              gh_scm2int(gh_list_ref(base, gh_int2scm(1))),
              gh_scm2int(gh_list_ref(base, gh_int2scm(2))),
              gh_scm2int(gh_list_ref(base, gh_int2scm(3))));
    CDAbsPos offset_;
    offset_.set(gh_scm2int(gh_list_ref(offset, gh_int2scm(0))),
                gh_scm2int(gh_list_ref(offset, gh_int2scm(1))),
                gh_scm2int(gh_list_ref(offset, gh_int2scm(2))));

    gh_defer_ints();
    CDRelPos ret=cd->add_pos(base_, offset_);
    gh_allow_ints();

    if(!ret.valid()) return SCM_BOOL_F;
    int pos_array[4];
    pos_array[0]=ret.track;
    pos_array[1]=ret.minute;
    pos_array[2]=ret.second;
    pos_array[3]=ret.frame;
    return gh_vector_to_list(gh_ints2scm(pos_array, 4));
}

SCM cdplayer_toc_changed()
{
    return (cd->toc_changed()) ? SCM_BOOL_T : SCM_BOOL_F;
}

SCM cdplayer_mixer_init()
{
    gh_defer_ints();
    mixer=new Mixer;
    gh_allow_ints();
    return SCM_UNSPECIFIED;
}    

SCM cdplayer_mixer_valid()
{
    gh_defer_ints();
    bool ret=mixer->valid();
    gh_allow_ints();
    return ret ? SCM_BOOL_T : SCM_BOOL_F;
}

SCM cdplayer_mixer_exit()
{
    gh_defer_ints();
    if(mixer) delete mixer;
    gh_allow_ints();
    mixer=0;
    return SCM_UNSPECIFIED;
}

SCM cdplayer_mixer_set_master_volume_front_left(SCM percent)
{
    gh_defer_ints();
    mixer->set_master_volume_front_left(gh_scm2double(percent));
    gh_allow_ints();
    return SCM_UNSPECIFIED;
}

SCM cdplayer_mixer_set_mute_front_left(SCM mute)
{
    gh_defer_ints();
    mixer->set_mute_front_left(gh_scm2bool(mute));
    gh_allow_ints();
    return SCM_UNSPECIFIED;
}

SCM cdplayer_mixer_set_master_volume_front_right(SCM percent)
{
    gh_defer_ints();
    mixer->set_master_volume_front_right(gh_scm2double(percent));
    gh_allow_ints();
    return SCM_UNSPECIFIED;
}

SCM cdplayer_mixer_set_mute_front_right(SCM mute)
{
    gh_defer_ints();
    mixer->set_mute_front_right(gh_scm2bool(mute));
    gh_allow_ints();
    return SCM_UNSPECIFIED;
}

void main_prog(int argc, char *argv[])
{
    atexit(cleanup);
    
    gh_new_procedure1_0("cdplayer-init", cdplayer_init);
    gh_new_procedure0_0("cdplayer-exit", cdplayer_exit);
    gh_new_procedure1_0("cdplayer-play", cdplayer_play);
    gh_new_procedure1_0("cdplayer-set-loop", cdplayer_set_loop);
    gh_new_procedure0_0("cdplayer-clear-loop", cdplayer_clear_loop);
    gh_new_procedure0_0("cdplayer-elapsed-gap", cdplayer_elapsed_gap);
    gh_new_procedure0_0("cdplayer-status", cdplayer_status);
    gh_new_procedure0_0("cdplayer-no-of-tracks", cdplayer_no_of_tracks);
    gh_new_procedure0_0("cdplayer-id", cdplayer_id);
    gh_new_procedure0_0("cdplayer-pos", cdplayer_pos);
    gh_new_procedure0_0("cdplayer-pause", cdplayer_pause);
    gh_new_procedure0_0("cdplayer-resume", cdplayer_resume);
    gh_new_procedure0_0("cdplayer-stop", cdplayer_stop);
    gh_new_procedure0_0("cdplayer-eject", cdplayer_eject);
    gh_new_procedure0_0("cdplayer-close-tray", cdplayer_close_tray);
    gh_new_procedure2_0("cdplayer-add-pos", cdplayer_add_pos);
    gh_new_procedure0_0("cdplayer-toc-changed", cdplayer_toc_changed);

    gh_new_procedure0_0("cdplayer-mixer-init", cdplayer_mixer_init);
    gh_new_procedure0_0("cdplayer-mixer-valid", cdplayer_mixer_valid);
    gh_new_procedure0_0("cdplayer-mixer-exit", cdplayer_mixer_exit);
    gh_new_procedure1_0("cdplayer-mixer-set-master-volume-front-left", 
                        cdplayer_mixer_set_master_volume_front_left);
    gh_new_procedure1_0("cdplayer-mixer-set-mute-front-left",
                        cdplayer_mixer_set_mute_front_left);
    gh_new_procedure1_0("cdplayer-mixer-set-master-volume-front-right", 
                        cdplayer_mixer_set_master_volume_front_right);
    gh_new_procedure1_0("cdplayer-mixer-set-mute-front-right",
                        cdplayer_mixer_set_mute_front_right);
    gh_eval_str(script);
 
    gh_repl(argc, argv);
}
     
int main(int argc, char *argv[])
{
    gh_enter(argc, argv, main_prog);
    return 0;  // never reached
}
